import {createProgramFromSource, ProgramError, ShaderError} from "./gl_util";

/// <reference path="./glsl.d.ts" />
import vert_src from "./vertex.vert";
import frag_src from "./fragment.frag";

import { getUniformDefaultValue, UniformSet } from "./uniform";

export class Renderer{
    gl : WebGL2RenderingContext;
    uniforms : UniformSet;
    program: WebGLProgram;
    positionBuffer: WebGLBuffer;
    indexBuffer: WebGLBuffer;
    constructor(gl: WebGL2RenderingContext){
        this.gl = gl;
        this.uniforms = getUniformDefaultValue();
        try{
            this.program = createProgramFromSource(gl,vert_src,frag_src);
            gl.useProgram(this.program);
        }
        catch(e){
            if(e instanceof ShaderError){
                console.log(e.message,"\n",e.info);
            }
            else if(e instanceof ProgramError){
                console.log(e.message,"\n",e.info);
            }
            else throw e;
        }
    }
    init(){
        const gl = this.gl;
        const position = [
            -0.5,-0.5,
            0.5,-0.5,
            0.5,0.5,
            -0.5,0.5
        ];
        const index = [
            0,1,2,
            2,3,0
        ];
        this.positionBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ARRAY_BUFFER,this.positionBuffer);
        gl.bufferData(gl.ARRAY_BUFFER,new Float32Array(position),gl.STATIC_DRAW);

        this.indexBuffer = gl.createBuffer();
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,this.indexBuffer);
        gl.bufferData(gl.ELEMENT_ARRAY_BUFFER,new Uint16Array(index),gl.STATIC_DRAW);
        this.settingUniform(this.uniforms);
    }
    useProgram(){
        this.gl.useProgram(this.program);
    }
    settingUniform(u:UniformSet){
        const gl = this.gl;
        this.uniforms = u;
        const location = gl.getUniformLocation(this.program, "u_color"); //u_color 변수 위치를 참조
        gl.uniform4f(location, this.uniforms.redcolor, 0.3, 0.8, 1.0); //해당 위치에 0.2, 0.3, 0.8, 1.0 데이터를 전달
    }
    draw(){
        const gl = this.gl;
        gl.clearColor(0,0,0,0);
        gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);

        gl.viewport(0,0,gl.canvas.width,gl.canvas.height);

        gl.enableVertexAttribArray(0);
        gl.vertexAttribPointer(0,2,gl.FLOAT,false,0,0);

        gl.drawElements(gl.TRIANGLES,6,gl.UNSIGNED_SHORT,0);
    }
}